library(tidyverse)
library(terra)
library(rnaturalearth)
library(furrr)

# Define Study extent ####
study_extent = rnaturalearth::ne_countries() %>% 
  dplyr::filter(continent == "South America") %>% 
  sf::st_union() %>% 
  sf::st_bbox() %>% 
  ext()

# CHELSA bioclim target variables ####
chelsa_urls = c("https://os.zhdk.cloud.switch.ch/chelsav2/GLOBAL/climatologies/1981-2010/bio/CHELSA_cmi_mean_1981-2010_V.2.1.tif",
                "https://os.zhdk.cloud.switch.ch/chelsav2/GLOBAL/climatologies/1981-2010/bio/CHELSA_rsds_1981-2010_mean_V.2.1.tif",
                "https://os.zhdk.cloud.switch.ch/chelsav2/GLOBAL/climatologies/1981-2010/bio/CHELSA_bio6_1981-2010_V.2.1.tif",
                "https://os.zhdk.cloud.switch.ch/chelsav2/GLOBAL/climatologies/1981-2010/bio/CHELSA_bio17_1981-2010_V.2.1.tif")

chelsa = terra::rast(chelsa_urls) %>% 
  crop(study_extent)
set.names(chelsa, c("cmi", "rsds", "bio6", "bio17"))
terra::writeRaster(chelsa, filename = file.path("data", "geospatial", "raster", basename(chelsa_urls)), overwrite = T)

# Tree canopy cover ####
igfc = list.files("I:/mas_data/00_data/processed/iGFC/", pattern = "canopyDensity_(199[0-9]|200[0-9]|2010)", full.names = TRUE) %>% 
  terra::rast() %>% 
  terra::resample(chelsa) %>%   
  app(mean, na.rm = TRUE)
terra::set.names(igfc, "igfc")
terra::writeRaster(igfc, filename = file.path("data", "geospatial", "raster", "mean_canopyDensity_1992-2010.tif"), overwrite = T)

# Seasonal inundation ####
igsw = list.files("I:/mas_data/00_data/processed/iGSW/", pattern = "seasonal_(199[0-9]|200[0-9]|2010)", full.names = TRUE) %>% 
  terra::rast() %>% 
  terra::resample(chelsa, threads = 32) %>%   
  app(mean, na.rm = TRUE) %>% 
  subst(NA, 0)
terra::set.names(igsw, "igsw")
terra::writeRaster(igsw, filename = file.path("data", "geospatial", "raster", "mean_seasonalInundation_1992-2010.tif"), overwrite = T)

# Distance to freshwater ####
dtfw = list.files("I:/mas_data/00_data/processed/linearDistance/", pattern = "lake|reservoirs|river", full.names = TRUE) %>% 
  terra::rast() %>% 
  terra::resample(chelsa) %>% 
  app(mean, na.rm = TRUE)
terra::set.names(dtfw, "dtfw")
terra::writeRaster(dtfw, filename = file.path("data", "geospatial", "raster", "mean_distanceToWater.tif"), overwrite = T)

# Terrain ####
roughness = list.files("C:/Users/ye87zine/Downloads", pattern = "roughness", full.names = T) %>% 
  terra::rast() %>% 
  crop(study_extent) %>% 
  terra::resample(chelsa)
terra::set.names(roughness, "roughness")
terra::writeRaster(roughness, filename = file.path("data", "geospatial", "raster", "terrain_roughness.tif"), overwrite = T)
