library(dplyr)
library(tidyr)
library(cito)

source("R/utils.R")

load("data/r_objects/model_data.RData")

model_data$species_int = as.integer(as.factor(model_data$species))

train_data = dplyr::filter(model_data, train == 1)
test_data = dplyr::filter(model_data, train == 0)

# ----------------------------------------------------------------------#
# Train model                                                        ####
# ----------------------------------------------------------------------#
predictors = paste0("layer_", 1:19)
formula = as.formula(paste0("present ~ ", paste(predictors, collapse = '+'), " + ", "e(species_int, dim = 50, lambda = 0.000001)"))

plot(1, type="n", xlab="", ylab="", xlim=c(0, 15000), ylim=c(0, 0.7)) # empty plot with better limits, draw points in there
msdm_fit_embedding_raw = dnn(
  formula,
  data = train_data,
  hidden = c(500L, 500L, 500L),
  loss = "binomial",
  activation = c("sigmoid", "leaky_relu", "leaky_relu"),
  epochs = 15000L, 
  lr = 0.01,   
  baseloss = 1,
  batchsize = nrow(train_data),
  dropout = 0.1,
  burnin = 100,
  optimizer = config_optimizer("adam", weight_decay = 0.001),
  lr_scheduler = config_lr_scheduler("reduce_on_plateau", patience = 100, factor = 0.7),
  early_stopping = 250,
  validation = 0.3,
  device = "cuda",
)

save(msdm_fit_embedding_raw, file = "data/r_objects/msdm_fit_embedding_raw.RData")

# ----------------------------------------------------------------------#
# Evaluate model                                                     ####
# ----------------------------------------------------------------------#
load("data/r_objects/msdm_fit_embedding_raw.RData")

data_split = split(model_data, model_data$species)

msdm_results_embedding_raw = lapply(data_split, function(data_spec){
  test_data = dplyr::filter(data_spec, train == 0) %>% 
    dplyr::select(-species)
  
  msdm_performance = tryCatch({
    evaluate_model(msdm_fit_embedding_raw, test_data)
  }, error = function(e){
    list(AUC = NA, Accuracy = NA, Kappa = NA, Precision = NA, Recall = NA, F1 = NA)
  })
  
  performance_summary = tibble(
    species = data_spec$species[1],
    obs = nrow(data_spec),
    model = "MSDM_embed",
    auc = msdm_performance$AUC,
    accuracy = msdm_performance$Accuracy,
    kappa = msdm_performance$Kappa,
    precision = msdm_performance$Precision,
    recall = msdm_performance$Recall,
    f1 = msdm_performance$F1
  )
}) %>% bind_rows()

save(msdm_results_embedding_raw, file = "data/r_objects/msdm_results_embedding_raw.RData")
